from typing import Any, Awaitable, Callable, Dict, Generic, Optional, TypeVar

from aiojobs_protocols import SchedulerProtocol

_T = TypeVar('_T')
_Context = Dict[str, Any]
_ExceptionHandler = Callable[[SchedulerProtocol, _Context], None]


class Job(Generic[_T]):
    active: bool
    pending: bool
    closed: bool
    async def wait(self, *, timeout: Optional[float] = None) -> _T: ...
    async def close(self, *, timeout: Optional[float] = None) -> None: ...


class Scheduler:
    limit: int
    pending_limit: int
    close_timeout: float
    active_count: int
    pending_count: int
    closed: bool
    exception_handler: _ExceptionHandler
    async def spawn(self, coro: Awaitable[_T]) -> Job[_T]: ...
    async def close(self) -> None: ...
    def call_exception_handler(self, context: _Context) -> None: ...


async def create_scheduler(
    close_timeout: float = ...,
    limit: int = ...,
    pending_limit: int = ...,
    exception_handler: Optional[_ExceptionHandler] = None
) -> Scheduler: ...



from typing import (Any, Awaitable, Callable, Dict, Optional, Protocol,
                    TypeVar, runtime_checkable)

_T = TypeVar('_T')
_T_co = TypeVar('_T_co', covariant=True)
_Context = Dict[str, Any]
_ExceptionHandler = Callable[['SchedulerProtocol', _Context], None]


@runtime_checkable
class JobProtocol(Protocol[_T_co]):
    active: bool
    pending: bool
    closed: bool
    async def wait(self, *, timeout: Optional[float] = None) -> _T_co: ...
    async def close(self, *, timeout: Optional[float] = None) -> None: ...


@runtime_checkable
class SchedulerProtocol(Protocol):
    limit: int
    pending_limit: int
    close_timeout: float
    active_count: int
    pending_count: int
    closed: bool
    exception_handler: _ExceptionHandler
    async def spawn(self, coro: Awaitable[_T]) -> JobProtocol[_T]: ...
    async def close(self) -> None: ...
    def call_exception_handler(self, context: _Context) -> None: ...



from pathlib import Path

from setuptools import setup

DIR = Path(__file__).parent

setup(
    name='aiojobs-stubs',
    author='Gleb Chipiga',
    version='1.0.0',
    description='External type annotations for the aiojobs library',
    long_description=(DIR / 'README.rst').read_text('utf-8').strip(),
    long_description_content_type='text/x-rst',
    url='https://github.com/gleb-chipiga/aiojobs-stubs',
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Framework :: AsyncIO'
    ],
    license='Apache 2',
    keywords=['asyncio', 'aiohttp', 'jobs', 'tasks', 'stubs', 'mypy'],
    packages=['aiojobs-stubs', 'aiojobs_protocols'],
    package_data={'aiojobs-stubs': ['__init__.pyi'],
                  'aiojobs_protocols': ['py.typed']},
    python_requires='>=3.8'
)
